@@ -1,23 +1,17 @@
 use crate::{
   activities::{
-    community::{announce::GetCommunity, send_activity_in_community},
+    community::announce::GetCommunity,
     deletion::{receive_delete_action, verify_delete_activity, DeletableObjects},
     generate_activity_id,
     verify_activity,
-    verify_is_public,
   },
-  activity_lists::AnnouncableActivities,
   objects::{community::ApubCommunity, person::ApubPerson},
-  protocol::activities::deletion::delete::Delete,
+  protocol::activities::deletion::delete::{Delete, IdOrNestedObject, NestedObject},
 };
-use activitystreams_kinds::{activity::DeleteType, public};
+use activitystreams_kinds::activity::DeleteType;
 use anyhow::anyhow;
 use lemmy_api_common::blocking;
-use lemmy_apub_lib::{
-  data::Data,
-  object_id::ObjectId,
-  traits::{ActivityHandler, ActorType},
-};
+use lemmy_apub_lib::{data::Data, object_id::ObjectId, traits::ActivityHandler};
 use lemmy_db_schema::{
   source::{
     comment::Comment,
@@ -30,6 +24,7 @@ use lemmy_db_schema::{
       ModRemovePost,
       ModRemovePostForm,
     },
+    person::Person,
     post::Post,
   },
   traits::Crud,
@@ -45,26 +40,19 @@ use url::Url;
 #[async_trait::async_trait(?Send)]
 impl ActivityHandler for Delete {
   type DataType = LemmyContext;
+
+  #[tracing::instrument(skip_all)]
   async fn verify(
     &self,
     context: &Data<LemmyContext>,
     request_counter: &mut i32,
   ) -> Result<(), LemmyError> {
-    verify_is_public(&self.to, &[])?;
     verify_activity(&self.id, self.actor.inner(), &context.settings())?;
-    let community = self.get_community(context, request_counter).await?;
-    verify_delete_activity(
-      &self.object.id,
-      &self.actor,
-      &community,
-      self.summary.is_some(),
-      context,
-      request_counter,
-    )
-    .await?;
+    verify_delete_activity(self, self.summary.is_some(), context, request_counter).await?;
     Ok(())
   }
 
+  #[tracing::instrument(skip_all)]
   async fn receive(
     self,
     context: &Data<LemmyContext>,
@@ -79,67 +67,72 @@ impl ActivityHandler for Delete {
         Some(reason)
       };
       receive_remove_action(
-        &self.actor,
-        &self.object.id,
+        &self
+          .actor
+          .dereference(context, context.client(), request_counter)
+          .await?,
+        self.object.id(),
         reason,
         context,
+      )
+      .await
+    } else {
+      receive_delete_action(
+        self.object.id(),
+        &self.actor,
+        true,
+        context,
         request_counter,
       )
       .await
-    } else {
-      receive_delete_action(&self.object.id, &self.actor, true, context, request_counter).await
     }
   }
 }
 
 impl Delete {
   pub(in crate::activities::deletion) fn new(
-    actor: &ApubPerson,
+    actor: &Person,
     object: DeletableObjects,
+    to: Url,
+    community: Option<&Community>,
     summary: Option<String>,
     context: &LemmyContext,
   ) -> Result<Delete, LemmyError> {
+    let id = generate_activity_id(
+      DeleteType::Delete,
+      &context.settings().get_protocol_and_hostname(),
+    )?;
+    let cc: Option<Url> = community.map(|c| c.actor_id.clone().into());
     Ok(Delete {
-      actor: ObjectId::new(actor.actor_id()),
-      to: vec![public()],
-      object: object.to_tombstone()?,
+      actor: ObjectId::new(actor.actor_id.clone()),
+      to: vec![to],
+      object: IdOrNestedObject::NestedObject(NestedObject {
+        id: object.id(),
+        kind: Default::default(),
+      }),
+      cc: cc.into_iter().collect(),
       kind: DeleteType::Delete,
       summary,
-      id: generate_activity_id(
-        DeleteType::Delete,
-        &context.settings().get_protocol_and_hostname(),
-      )?,
+      id,
       unparsed: Default::default(),
     })
   }
-  pub(in crate::activities::deletion) async fn send(
-    actor: &ApubPerson,
-    community: &ApubCommunity,
-    object: DeletableObjects,
-    summary: Option<String>,
-    context: &LemmyContext,
-  ) -> Result<(), LemmyError> {
-    let delete = Delete::new(actor, object, summary, context)?;
-    let delete_id = delete.id.clone();
-
-    let activity = AnnouncableActivities::Delete(delete);
-    send_activity_in_community(activity, &delete_id, actor, community, vec![], context).await
-  }
 }
 
+#[tracing::instrument(skip_all)]
 pub(in crate::activities) async fn receive_remove_action(
-  actor: &ObjectId<ApubPerson>,
+  actor: &ApubPerson,
   object: &Url,
   reason: Option<String>,
   context: &LemmyContext,
-  request_counter: &mut i32,
 ) -> Result<(), LemmyError> {
-  let actor = actor.dereference(context, request_counter).await?;
   use UserOperationCrud::*;
   match DeletableObjects::read_from_db(object, context).await? {
     DeletableObjects::Community(community) => {
       if community.local {
-        return Err(anyhow!("Only local admin can remove community").into());
+        return Err(LemmyError::from_message(
+          "Only local admin can remove community",
+        ));
       }
       let form = ModRemoveCommunityForm {
         mod_person_id: actor.id,
@@ -195,24 +188,29 @@ pub(in crate::activities) async fn receive_remove_action(
 
       send_comment_ws_message_simple(removed_comment.id, RemoveComment, context).await?;
     }
+    DeletableObjects::PrivateMessage(_) => unimplemented!(),
   }
   Ok(())
 }
 
 #[async_trait::async_trait(?Send)]
 impl GetCommunity for Delete {
+  #[tracing::instrument(skip_all)]
   async fn get_community(
     &self,
     context: &LemmyContext,
     _request_counter: &mut i32,
   ) -> Result<ApubCommunity, LemmyError> {
-    let community_id = match DeletableObjects::read_from_db(&self.object.id, context).await? {
+    let community_id = match DeletableObjects::read_from_db(self.object.id(), context).await? {
       DeletableObjects::Community(c) => c.id,
       DeletableObjects::Comment(c) => {
         let post = blocking(context.pool(), move |conn| Post::read(conn, c.post_id)).await??;
         post.community_id
       }
       DeletableObjects::Post(p) => p.community_id,
+      DeletableObjects::PrivateMessage(_) => {
+        return Err(anyhow!("Private message is not part of community").into())
+      }
     };
     let community = blocking(context.pool(), move |conn| {
       Community::read(conn, community_id)
