@@ -13,6 +13,7 @@ use crate::{
   PostOrComment,
 };
 use activitystreams_kinds::public;
+use anyhow::anyhow;
 use lemmy_api_common::blocking;
 use lemmy_apub_lib::{
   data::Data,
@@ -21,12 +22,14 @@ use lemmy_apub_lib::{
 };
 use lemmy_db_schema::{
   newtypes::CommunityId,
-  source::{community::Community, post::Post},
+  source::{community::Community, post::Post, site::Site},
   traits::Crud,
 };
 use lemmy_utils::LemmyError;
 use lemmy_websocket::LemmyContext;
 
+/// Vote has as:Public value in cc field, unlike other activities. This indicates to other software
+/// (like GNU social, or presumably Mastodon), that the like actor should not be disclosed.
 impl Vote {
   pub(in crate::activities::voting) fn new(
     object: &PostOrComment,
@@ -37,9 +40,9 @@ impl Vote {
   ) -> Result<Vote, LemmyError> {
     Ok(Vote {
       actor: ObjectId::new(actor.actor_id()),
-      to: vec![public()],
+      to: vec![community.actor_id()],
       object: ObjectId::new(object.ap_id()),
-      cc: vec![community.actor_id()],
+      cc: vec![public()],
       kind: kind.clone(),
       id: generate_activity_id(kind, &context.settings().get_protocol_and_hostname())?,
       unparsed: Default::default(),
@@ -81,6 +84,10 @@ impl ActivityHandler for Vote {
     verify_activity(&self.id, self.actor.inner(), &context.settings())?;
     let community = self.get_community(context, request_counter).await?;
     verify_person_in_community(&self.actor, &community, context, request_counter).await?;
+    let site = blocking(context.pool(), Site::read_local_site).await??;
+    if self.kind == VoteType::Dislike && !site.enable_downvotes {
+      return Err(anyhow!("Downvotes disabled").into());
+    }
     Ok(())
   }
 
