@@ -18,7 +18,7 @@ pub enum PostOrComment {
 #[derive(Deserialize)]
 #[serde(untagged)]
 pub enum PageOrNote {
-  Page(Page),
+  Page(Box<Page>),
   Note(Note),
 }
 
@@ -26,6 +26,7 @@ pub enum PageOrNote {
 impl ApubObject for PostOrComment {
   type DataType = LemmyContext;
   type ApubType = PageOrNote;
+  type DbType = ();
   type TombstoneType = ();
 
   fn last_refreshed_at(&self) -> Option<NaiveDateTime> {
@@ -84,7 +85,7 @@ impl ApubObject for PostOrComment {
   ) -> Result<Self, LemmyError> {
     Ok(match apub {
       PageOrNote::Page(p) => PostOrComment::Post(Box::new(
-        ApubPost::from_apub(p, context, request_counter).await?,
+        ApubPost::from_apub(*p, context, request_counter).await?,
       )),
       PageOrNote::Note(n) => PostOrComment::Comment(Box::new(
         ApubComment::from_apub(n, context, request_counter).await?,
