@@ -1,16 +1,12 @@
 use crate::{
+  activity_lists::SharedInboxActivities,
   check_is_apub_id_valid,
-  extensions::signatures::verify_signature,
-  fetcher::get_or_fetch_and_upsert_actor,
-  http::{
-    community::{receive_group_inbox, GroupInboxActivities},
-    person::{receive_person_inbox, PersonInboxActivities},
-  },
+  context::WithContext,
+  fetcher::user_or_community::UserOrCommunity,
+  http::{community::receive_group_inbox, person::receive_person_inbox},
   insert_activity,
-  APUB_JSON_CONTENT_TYPE,
 };
 use actix_web::{
-  body::Body,
   web,
   web::{Bytes, BytesMut, Payload},
   HttpRequest,
@@ -20,14 +16,19 @@ use anyhow::{anyhow, Context};
 use futures::StreamExt;
 use http::StatusCode;
 use lemmy_api_common::blocking;
-use lemmy_apub_lib::{ActivityFields, ActivityHandler};
-use lemmy_db_queries::{source::activity::Activity_, DbPool};
+use lemmy_apub_lib::{
+  data::Data,
+  object_id::ObjectId,
+  signatures::verify_signature,
+  traits::{ActivityHandler, ActorType},
+  APUB_JSON_CONTENT_TYPE,
+};
 use lemmy_db_schema::source::activity::Activity;
 use lemmy_utils::{location_info, LemmyError};
 use lemmy_websocket::LemmyContext;
-use log::{info, trace};
 use serde::{Deserialize, Serialize};
 use std::{fmt::Debug, io::Read};
+use tracing::{debug, info};
 use url::Url;
 
 mod comment;
@@ -35,30 +36,24 @@ mod community;
 mod person;
 mod post;
 pub mod routes;
+pub mod site;
 
-#[derive(Clone, Debug, Deserialize, Serialize, ActivityHandler, ActivityFields)]
-#[serde(untagged)]
-pub enum SharedInboxActivities {
-  GroupInboxActivities(GroupInboxActivities),
-  // Note, pm activities need to be at the end, otherwise comments will end up here. We can probably
-  // avoid this problem by replacing createpm.object with our own struct, instead of NoteExt.
-  PersonInboxActivities(PersonInboxActivities),
-}
-
+#[tracing::instrument(skip_all)]
 pub async fn shared_inbox(
   request: HttpRequest,
   payload: Payload,
   context: web::Data<LemmyContext>,
 ) -> Result<HttpResponse, LemmyError> {
   let unparsed = payload_to_string(payload).await?;
-  trace!("Received shared inbox activity {}", unparsed);
-  let activity = serde_json::from_str::<SharedInboxActivities>(&unparsed)?;
-  match activity {
+  info!("Received shared inbox activity {}", unparsed);
+  let activity_data: ActivityCommonFields = serde_json::from_str(&unparsed)?;
+  let activity = serde_json::from_str::<WithContext<SharedInboxActivities>>(&unparsed)?;
+  match activity.inner() {
     SharedInboxActivities::GroupInboxActivities(g) => {
-      receive_group_inbox(g, request, &context).await
+      receive_group_inbox(*g, activity_data, request, &context).await
     }
     SharedInboxActivities::PersonInboxActivities(p) => {
-      receive_person_inbox(p, request, &context).await
+      receive_person_inbox(*p, activity_data, request, &context).await
     }
   }
 }
@@ -73,15 +68,23 @@ async fn payload_to_string(mut payload: Payload) -> Result<String, LemmyError> {
   Ok(unparsed)
 }
 
+#[derive(Clone, Debug, Deserialize, Serialize)]
+#[serde(rename_all = "camelCase")]
+pub(crate) struct ActivityCommonFields {
+  pub(crate) id: Url,
+  pub(crate) actor: Url,
+}
+
 // TODO: move most of this code to library
+#[tracing::instrument(skip_all)]
 async fn receive_activity<'a, T>(
   request: HttpRequest,
   activity: T,
+  activity_data: ActivityCommonFields,
   context: &LemmyContext,
 ) -> Result<HttpResponse, LemmyError>
 where
-  T: ActivityHandler
-    + ActivityFields
+  T: ActivityHandler<DataType = LemmyContext>
     + Clone
     + Deserialize<'a>
     + Serialize
@@ -89,55 +92,64 @@ where
     + Send
     + 'static,
 {
+  check_is_apub_id_valid(&activity_data.actor, false, &context.settings())?;
   let request_counter = &mut 0;
-  let actor =
-    get_or_fetch_and_upsert_actor(activity.actor().clone(), context, request_counter).await?;
-  verify_signature(&request, &actor.public_key().context(location_info!())?)?;
+  let actor = ObjectId::<UserOrCommunity>::new(activity_data.actor)
+    .dereference(context, context.client(), request_counter)
+    .await?;
+  verify_signature(&request, &actor.public_key())?;
 
-  // Do nothing if we received the same activity before
-  if is_activity_already_known(context.pool(), activity.id_unchecked()).await? {
-    return Ok(HttpResponse::Ok().finish());
-  }
-  check_is_apub_id_valid(activity.actor(), false, &context.settings())?;
-  info!("Verifying activity {}", activity.id_unchecked().to_string());
-  activity.verify(context, request_counter).await?;
-  assert_activity_not_local(&activity, &context.settings().hostname)?;
+  info!("Verifying activity {}", activity_data.id.to_string());
+  activity
+    .verify(&Data::new(context.clone()), request_counter)
+    .await?;
+  assert_activity_not_local(&activity_data.id, &context.settings().hostname)?;
 
   // Log the activity, so we avoid receiving and parsing it twice. Note that this could still happen
   // if we receive the same activity twice in very quick succession.
-  insert_activity(
-    activity.id_unchecked(),
-    activity.clone(),
-    false,
-    true,
-    context.pool(),
-  )
-  .await?;
+  let object_value = serde_json::to_value(&activity)?;
+  let insert =
+    insert_activity(&activity_data.id, object_value, false, true, context.pool()).await?;
+  if !insert {
+    debug!(
+      "Received duplicate activity {}",
+      activity_data.id.to_string()
+    );
+    return Ok(HttpResponse::BadRequest().finish());
+  }
 
-  info!("Receiving activity {}", activity.id_unchecked().to_string());
-  activity.receive(context, request_counter).await?;
+  info!("Receiving activity {}", activity_data.id.to_string());
+  activity
+    .receive(&Data::new(context.clone()), request_counter)
+    .await?;
   Ok(HttpResponse::Ok().finish())
 }
 
 /// Convert the data to json and turn it into an HTTP Response with the correct ActivityPub
 /// headers.
-fn create_apub_response<T>(data: &T) -> HttpResponse<Body>
+fn create_apub_response<T>(data: &T) -> HttpResponse
 where
   T: Serialize,
 {
+  HttpResponse::Ok()
+    .content_type(APUB_JSON_CONTENT_TYPE)
+    .json(WithContext::new(data))
+}
+
+fn create_json_apub_response(data: serde_json::Value) -> HttpResponse {
   HttpResponse::Ok()
     .content_type(APUB_JSON_CONTENT_TYPE)
     .json(data)
 }
 
-fn create_apub_tombstone_response<T>(data: &T) -> HttpResponse<Body>
+fn create_apub_tombstone_response<T>(data: &T) -> HttpResponse
 where
   T: Serialize,
 {
   HttpResponse::Gone()
     .content_type(APUB_JSON_CONTENT_TYPE)
     .status(StatusCode::GONE)
-    .json(data)
+    .json(WithContext::new(data))
 }
 
 #[derive(Deserialize)]
@@ -147,10 +159,11 @@ pub struct ActivityQuery {
 }
 
 /// Return the ActivityPub json representation of a local activity over HTTP.
+#[tracing::instrument(skip_all)]
 pub(crate) async fn get_activity(
   info: web::Path<ActivityQuery>,
   context: web::Data<LemmyContext>,
-) -> Result<HttpResponse<Body>, LemmyError> {
+) -> Result<HttpResponse, LemmyError> {
   let settings = context.settings();
   let activity_id = Url::parse(&format!(
     "{}/activities/{}/{}",
@@ -168,39 +181,22 @@ pub(crate) async fn get_activity(
   if !activity.local || sensitive {
     Ok(HttpResponse::NotFound().finish())
   } else {
-    Ok(create_apub_response(&activity.data))
+    Ok(create_json_apub_response(activity.data))
   }
 }
 
-pub(crate) async fn is_activity_already_known(
-  pool: &DbPool,
-  activity_id: &Url,
-) -> Result<bool, LemmyError> {
-  let activity_id = activity_id.to_owned().into();
-  let existing = blocking(pool, move |conn| {
-    Activity::read_from_apub_id(conn, &activity_id)
-  })
-  .await?;
-  match existing {
-    Ok(_) => Ok(true),
-    Err(_) => Ok(false),
-  }
-}
-
-fn assert_activity_not_local<T: Debug + ActivityFields>(
-  activity: &T,
-  hostname: &str,
-) -> Result<(), LemmyError> {
-  let activity_domain = activity.id_unchecked().domain().context(location_info!())?;
+fn assert_activity_not_local(id: &Url, hostname: &str) -> Result<(), LemmyError> {
+  let activity_domain = id.domain().context(location_info!())?;
 
   if activity_domain == hostname {
-    return Err(
-      anyhow!(
-        "Error: received activity which was sent by local instance: {:?}",
-        activity
-      )
-      .into(),
+    let err = anyhow!(
+      "Error: received activity which was sent by local instance: {:?}",
+      id
     );
+    return Err(LemmyError::from_error_message(
+      err,
+      "received_local_activity",
+    ));
   }
   Ok(())
 }
