@@ -1,6 +1,6 @@
 use crate::{
   check_is_apub_id_valid,
-  objects::get_summary_from_string_or_source,
+  objects::{read_from_string_or_source_opt, verify_image_domain_matches},
   protocol::{objects::instance::Instance, ImageObject, Source},
 };
 use activitystreams_kinds::actor::ServiceType;
@@ -37,7 +37,7 @@ impl Deref for ApubSite {
 
 impl From<Site> for ApubSite {
   fn from(s: Site) -> Self {
-    ApubSite { 0: s }
+    ApubSite(s)
   }
 }
 
@@ -45,6 +45,7 @@ impl From<Site> for ApubSite {
 impl ApubObject for ApubSite {
   type DataType = LemmyContext;
   type ApubType = Instance;
+  type DbType = Site;
   type TombstoneType = ();
 
   fn last_refreshed_at(&self) -> Option<NaiveDateTime> {
@@ -103,6 +104,8 @@ impl ApubObject for ApubSite {
   ) -> Result<(), LemmyError> {
     check_is_apub_id_valid(apub.id.inner(), true, &data.settings())?;
     verify_domains_match(expected_domain, apub.id.inner())?;
+    verify_image_domain_matches(expected_domain, &apub.icon)?;
+    verify_image_domain_matches(expected_domain, &apub.image)?;
 
     let slur_regex = &data.settings().slur_regex();
     check_slurs(&apub.name, slur_regex)?;
@@ -118,10 +121,7 @@ impl ApubObject for ApubSite {
   ) -> Result<Self, LemmyError> {
     let site_form = SiteForm {
       name: apub.name.clone(),
-      sidebar: Some(get_summary_from_string_or_source(
-        &apub.content,
-        &apub.source,
-      )),
+      sidebar: Some(read_from_string_or_source_opt(&apub.content, &apub.source)),
       updated: apub.updated.map(|u| u.clone().naive_local()),
       icon: Some(apub.icon.clone().map(|i| i.url.into())),
       banner: Some(apub.image.clone().map(|i| i.url.into())),
