@@ -1,9 +1,10 @@
 use crate::{
   activities::{verify_is_public, verify_person_in_community},
   check_is_apub_id_valid,
+  objects::read_from_string_or_source_opt,
   protocol::{
     objects::{
-      page::{Page, PageType},
+      page::{Attachment, Page, PageType},
       tombstone::Tombstone,
     },
     ImageObject,
@@ -16,7 +17,7 @@ use lemmy_api_common::blocking;
 use lemmy_apub_lib::{
   object_id::ObjectId,
   traits::ApubObject,
-  values::{MediaTypeHtml, MediaTypeMarkdown},
+  values::MediaTypeHtml,
   verify::verify_domains_match,
 };
 use lemmy_db_schema::{
@@ -49,7 +50,7 @@ impl Deref for ApubPost {
 
 impl From<Post> for ApubPost {
   fn from(p: Post) -> Self {
-    ApubPost { 0: p }
+    ApubPost(p)
   }
 }
 
@@ -57,6 +58,7 @@ impl From<Post> for ApubPost {
 impl ApubObject for ApubPost {
   type DataType = LemmyContext;
   type ApubType = Page;
+  type DbType = Post;
   type TombstoneType = Tombstone;
 
   fn last_refreshed_at(&self) -> Option<NaiveDateTime> {
@@ -99,12 +101,6 @@ impl ApubObject for ApubPost {
     })
     .await??;
 
-    let source = self.body.clone().map(|body| Source {
-      content: body,
-      media_type: MediaTypeMarkdown::Markdown,
-    });
-    let image = self.thumbnail_url.clone().map(ImageObject::new);
-
     let page = Page {
       r#type: PageType::Page,
       id: ObjectId::new(self.ap_id.clone()),
@@ -114,9 +110,10 @@ impl ApubObject for ApubPost {
       name: self.name.clone(),
       content: self.body.as_ref().map(|b| markdown_to_html(b)),
       media_type: Some(MediaTypeHtml::Html),
-      source,
+      source: self.body.clone().map(Source::new),
       url: self.url.clone().map(|u| u.into()),
-      image,
+      attachment: self.url.clone().map(Attachment::new).into_iter().collect(),
+      image: self.thumbnail_url.clone().map(ImageObject::new),
       comments_enabled: Some(!self.locked),
       sensitive: Some(self.nsfw),
       stickied: Some(self.stickied),
@@ -164,8 +161,15 @@ impl ApubObject for ApubPost {
       .await?;
     let community = page.extract_community(context, request_counter).await?;
 
+    // TODO: write mod log if stickied or locked changed
+
+    let url = if let Some(attachment) = page.attachment.first() {
+      Some(attachment.href.clone())
+    } else {
+      page.url
+    };
     let thumbnail_url: Option<Url> = page.image.map(|i| i.url);
-    let (metadata_res, pictrs_thumbnail) = if let Some(url) = &page.url {
+    let (metadata_res, pictrs_thumbnail) = if let Some(url) = &url {
       fetch_site_data(context.client(), &context.settings(), Some(url)).await
     } else {
       (None, thumbnail_url)
@@ -174,13 +178,11 @@ impl ApubObject for ApubPost {
       .map(|u| (u.title, u.description, u.html))
       .unwrap_or((None, None, None));
 
-    let body_slurs_removed = page
-      .source
-      .as_ref()
-      .map(|s| remove_slurs(&s.content, &context.settings().slur_regex()));
+    let body_slurs_removed = read_from_string_or_source_opt(&page.content, &page.source)
+      .map(|s| remove_slurs(&s, &context.settings().slur_regex()));
     let form = PostForm {
-      name: page.name,
-      url: page.url.map(|u| u.into()),
+      name: page.name.clone(),
+      url: url.map(Into::into),
       body: body_slurs_removed,
       creator_id: creator.id,
       community_id: community.id,
