@@ -21,6 +21,7 @@ mod tests {
       create_or_update::{comment::CreateOrUpdateComment, post::CreateOrUpdatePost},
       deletion::delete::Delete,
       following::{follow::FollowCommunity, undo_follow::UndoFollowCommunity},
+      voting::{undo_vote::UndoVote, vote::Vote},
     },
     tests::test_json,
   };
@@ -53,6 +54,20 @@ mod tests {
 
   #[test]
   fn test_parse_friendica_activities() {
+    test_json::<CreateOrUpdatePost>("assets/friendica/activities/create_page_1.json").unwrap();
+    test_json::<CreateOrUpdatePost>("assets/friendica/activities/create_page_2.json").unwrap();
     test_json::<CreateOrUpdateComment>("assets/friendica/activities/create_note.json").unwrap();
+    test_json::<CreateOrUpdateComment>("assets/friendica/activities/update_note.json").unwrap();
+    test_json::<Delete>("assets/friendica/activities/delete.json").unwrap();
+    test_json::<Vote>("assets/friendica/activities/like_page.json").unwrap();
+    test_json::<Vote>("assets/friendica/activities/dislike_page.json").unwrap();
+    test_json::<UndoVote>("assets/friendica/activities/undo_dislike_page.json").unwrap();
+  }
+
+  #[test]
+  fn test_parse_gnusocial_activities() {
+    test_json::<CreateOrUpdatePost>("assets/gnusocial/activities/create_page.json").unwrap();
+    test_json::<CreateOrUpdateComment>("assets/gnusocial/activities/create_note.json").unwrap();
+    test_json::<Vote>("assets/gnusocial/activities/like_note.json").unwrap();
   }
 }
