@@ -1,10 +1,9 @@
 use activitystreams_kinds::object::ImageType;
-use serde::{Deserialize, Serialize};
-use url::Url;
-
 use lemmy_apub_lib::values::MediaTypeMarkdown;
 use lemmy_db_schema::newtypes::DbUrl;
+use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
+use url::Url;
 
 pub mod activities;
 pub(crate) mod collections;
@@ -17,6 +16,15 @@ pub struct Source {
   pub(crate) media_type: MediaTypeMarkdown,
 }
 
+impl Source {
+  pub(crate) fn new(content: String) -> Self {
+    Source {
+      content,
+      media_type: MediaTypeMarkdown::Markdown,
+    }
+  }
+}
+
 #[derive(Clone, Debug, Deserialize, Serialize)]
 #[serde(rename_all = "camelCase")]
 pub struct ImageObject {
@@ -40,11 +48,21 @@ pub struct Unparsed(HashMap<String, serde_json::Value>);
 
 #[cfg(test)]
 pub(crate) mod tests {
-  use crate::objects::tests::file_to_json_object;
+  use crate::context::WithContext;
   use assert_json_diff::assert_json_include;
   use lemmy_utils::LemmyError;
   use serde::{de::DeserializeOwned, Serialize};
-  use std::collections::HashMap;
+  use std::{collections::HashMap, fs::File, io::BufReader};
+
+  pub(crate) fn file_to_json_object<T: DeserializeOwned>(path: &str) -> Result<T, LemmyError> {
+    let file = File::open(path)?;
+    let reader = BufReader::new(file);
+    Ok(serde_json::from_reader(reader)?)
+  }
+
+  pub(crate) fn test_json<T: DeserializeOwned>(path: &str) -> Result<WithContext<T>, LemmyError> {
+    file_to_json_object::<WithContext<T>>(path)
+  }
 
   /// Check that json deserialize -> serialize -> deserialize gives identical file as initial one.
   /// Ensures that there are no breaking changes in sent data.
