@@ -4,8 +4,12 @@ use crate::{
     community_moderators::ApubCommunityModerators,
     community_outbox::ApubCommunityOutbox,
   },
-  objects::{community::ApubCommunity, get_summary_from_string_or_source},
-  protocol::{objects::Endpoints, ImageObject, Source, Unparsed},
+  objects::{
+    community::ApubCommunity,
+    read_from_string_or_source_opt,
+    verify_image_domain_matches,
+  },
+  protocol::{objects::Endpoints, ImageObject, Source},
 };
 use activitystreams_kinds::actor::GroupType;
 use chrono::{DateTime, FixedOffset};
@@ -27,11 +31,17 @@ pub struct Group {
   #[serde(rename = "type")]
   pub(crate) kind: GroupType,
   pub(crate) id: ObjectId<ApubCommunity>,
-  /// username, set at account creation and can never be changed
+  /// username, set at account creation and usually fixed after that
   pub(crate) preferred_username: String,
-  /// title (can be changed at any time)
-  pub(crate) name: String,
+  pub(crate) inbox: Url,
+  pub(crate) followers: Url,
+  pub(crate) public_key: PublicKey,
+
+  /// title
+  pub(crate) name: Option<String>,
   pub(crate) summary: Option<String>,
+  #[serde(default)]
+  #[serde(deserialize_with = "crate::deserialize_skip_error")]
   pub(crate) source: Option<Source>,
   pub(crate) icon: Option<ImageObject>,
   /// banner
@@ -40,15 +50,10 @@ pub struct Group {
   pub(crate) sensitive: Option<bool>,
   // lemmy extension
   pub(crate) moderators: Option<ObjectId<ApubCommunityModerators>>,
-  pub(crate) inbox: Url,
   pub(crate) outbox: ObjectId<ApubCommunityOutbox>,
-  pub(crate) followers: Url,
   pub(crate) endpoints: Option<Endpoints>,
-  pub(crate) public_key: PublicKey,
   pub(crate) published: Option<DateTime<FixedOffset>>,
   pub(crate) updated: Option<DateTime<FixedOffset>>,
-  #[serde(flatten)]
-  pub(crate) unparsed: Unparsed,
 }
 
 impl Group {
@@ -59,20 +64,22 @@ impl Group {
   ) -> Result<(), LemmyError> {
     check_is_apub_id_valid(self.id.inner(), true, &context.settings())?;
     verify_domains_match(expected_domain, self.id.inner())?;
+    verify_image_domain_matches(expected_domain, &self.icon)?;
+    verify_image_domain_matches(expected_domain, &self.image)?;
 
     let slur_regex = &context.settings().slur_regex();
     check_slurs(&self.preferred_username, slur_regex)?;
-    check_slurs(&self.name, slur_regex)?;
-    let description = get_summary_from_string_or_source(&self.summary, &self.source);
+    check_slurs_opt(&self.name, slur_regex)?;
+    let description = read_from_string_or_source_opt(&self.summary, &self.source);
     check_slurs_opt(&description, slur_regex)?;
     Ok(())
   }
 
   pub(crate) fn into_form(self) -> CommunityForm {
     CommunityForm {
-      name: self.preferred_username,
-      title: self.name,
-      description: get_summary_from_string_or_source(&self.summary, &self.source),
+      name: self.preferred_username.clone(),
+      title: self.name.unwrap_or(self.preferred_username),
+      description: read_from_string_or_source_opt(&self.summary, &self.source),
       removed: None,
       published: self.published.map(|u| u.naive_local()),
       updated: self.updated.map(|u| u.naive_local()),
@@ -81,6 +88,7 @@ impl Group {
       actor_id: Some(self.id.into()),
       local: Some(false),
       private_key: None,
+      hidden: Some(false),
       public_key: self.public_key.public_key_pem,
       last_refreshed_at: Some(naive_now()),
       icon: Some(self.icon.map(|i| i.url.into())),
